@@ -1,10 +1,12 @@
 import asyncio
 import importlib
+import sys
 import time
 from functools import wraps
+from getpass import getpass
 from pathlib import Path
-from typing import List, Optional, Tuple
-from urllib.parse import urlparse
+from typing import Optional
+from uuid import uuid4
 
 import click
 import httpx
@@ -12,37 +14,36 @@ from fastapi.exceptions import HTTPException
 from loguru import logger
 from packaging import version
 
-from lnbits.core.models import Payment, User
-from lnbits.core.services import check_admin_settings
+from lnbits.core import db as core_db
+from lnbits.core.crud import (
+    delete_accounts_no_wallets,
+    delete_unused_wallets,
+    delete_wallet_by_id,
+    delete_wallet_payment,
+    get_db_versions,
+    get_installed_extension,
+    get_installed_extensions,
+    get_payment,
+    get_payments,
+    remove_deleted_wallets,
+    update_payment,
+)
+from lnbits.core.helpers import is_valid_url, migrate_databases
+from lnbits.core.models import Account, Payment, PaymentState
+from lnbits.core.models.extensions import (
+    CreateExtension,
+    ExtensionRelease,
+    InstallableExtension,
+)
+from lnbits.core.services import check_admin_settings, create_user_account_no_ckeck
 from lnbits.core.views.extension_api import (
     api_install_extension,
     api_uninstall_extension,
 )
 from lnbits.settings import settings
+from lnbits.utils.crypto import AESCipher
 from lnbits.wallets.base import Wallet
-
-from .core import db as core_db
-from .core import migrations as core_migrations
-from .core.crud import (
-    delete_accounts_no_wallets,
-    delete_unused_wallets,
-    delete_wallet_by_id,
-    delete_wallet_payment,
-    get_dbversions,
-    get_installed_extension,
-    get_installed_extensions,
-    get_payments,
-    remove_deleted_wallets,
-    update_payment_status,
-)
-from .core.helpers import migrate_extension_database, run_migration
-from .db import COCKROACH, POSTGRES, SQLITE
-from .extension_manager import (
-    CreateExtension,
-    ExtensionRelease,
-    InstallableExtension,
-    get_valid_extensions,
-)
+from lnbits.wallets.macaroon import load_macaroon
 
 
 def coro(f):
@@ -67,6 +68,13 @@ def db():
     """
 
 
+@lnbits_cli.group()
+def users():
+    """
+    Users related commands
+    """
+
+
 @lnbits_cli.group()
 def extensions():
     """
@@ -74,6 +82,20 @@ def extensions():
     """
 
 
+@lnbits_cli.group()
+def encrypt():
+    """
+    Encryption commands
+    """
+
+
+@lnbits_cli.group()
+def decrypt():
+    """
+    Decryption commands
+    """
+
+
 def get_super_user() -> Optional[str]:
     """Get the superuser"""
     superuser_file = Path(settings.lnbits_data_folder, ".super_user")
@@ -112,7 +134,7 @@ async def delete_settings():
     """Deletes the settings"""
 
     async with core_db.connect() as conn:
-        await conn.execute("DELETE from settings")
+        await conn.execute("DELETE from system_settings")
 
 
 @db.command("migrate")
@@ -122,54 +144,12 @@ def database_migrate():
     loop.run_until_complete(migrate_databases())
 
 
-async def db_migrate():
-    task = asyncio.create_task(migrate_databases())
-    await task
-
-
-async def migrate_databases():
-    """Creates the necessary databases if they don't exist already; or migrates them."""
-
-    async with core_db.connect() as conn:
-        exists = False
-        if conn.type == SQLITE:
-            exists = await conn.fetchone(
-                "SELECT * FROM sqlite_master WHERE type='table' AND name='dbversions'"
-            )
-        elif conn.type in {POSTGRES, COCKROACH}:
-            exists = await conn.fetchone(
-                "SELECT * FROM information_schema.tables WHERE table_schema = 'public'"
-                " AND table_name = 'dbversions'"
-            )
-
-        if not exists:
-            await core_migrations.m000_create_migrations_table(conn)
-
-        current_versions = await get_dbversions(conn)
-        core_version = current_versions.get("core", 0)
-        await run_migration(conn, core_migrations, "core", core_version)
-
-    # here is the first place we can be sure that the
-    # `installed_extensions` table has been created
-    await load_disabled_extension_list()
-
-    # todo: revisit, use installed extensions
-    for ext in get_valid_extensions(False):
-        current_version = current_versions.get(ext.code, 0)
-        try:
-            await migrate_extension_database(ext, current_version)
-        except Exception as e:
-            logger.exception(f"Error migrating extension {ext.code}: {e}")
-
-    logger.info("✔️ All migrations done.")
-
-
 @db.command("versions")
 @coro
 async def db_versions():
     """Show current database versions"""
     async with core_db.connect() as conn:
-        click.echo(await get_dbversions(conn))
+        click.echo(await get_db_versions(conn))
 
 
 @db.command("cleanup-wallets")
@@ -216,21 +196,13 @@ async def database_delete_wallet_payment(wallet: str, checking_id: str):
 @db.command("mark-payment-pending")
 @click.option("-c", "--checking-id", required=True, help="Payment checking Id.")
 @coro
-async def database_revert_payment(checking_id: str, pending: bool = True):
-    """Mark wallet as deleted"""
+async def database_revert_payment(checking_id: str):
+    """Mark payment as pending"""
     async with core_db.connect() as conn:
-        await update_payment_status(pending=pending, checking_id=checking_id, conn=conn)
-
-
-@db.command("cleanup-accounts")
-@click.argument("days", type=int, required=False)
-@coro
-async def database_cleanup_accounts(days: Optional[int] = None):
-    """Delete all accounts that have no wallets"""
-    async with core_db.connect() as conn:
-        delta = days or settings.cleanup_wallets_days
-        delta = delta * 24 * 60 * 60
-        await delete_accounts_no_wallets(delta, conn)
+        payment = await get_payment(checking_id=checking_id, conn=conn)
+        payment.status = PaymentState.PENDING
+        await update_payment(payment, conn=conn)
+        click.echo(f"Payment '{checking_id}' marked as pending.")
 
 
 @db.command("check-payments")
@@ -276,7 +248,7 @@ async def check_invalid_payments(
     click.echo("Funding source: " + str(funding_source))
 
     # payments that are settled in the DB, but not at the Funding source level
-    invalid_payments: List[Payment] = []
+    invalid_payments: list[Payment] = []
     invalid_wallets = {}
     for db_payment in settled_db_payments:
         if verbose:
@@ -313,10 +285,30 @@ async def check_invalid_payments(
         click.echo(" ".join([w, str(data[0]), str(data[1] / 1000).ljust(10)]))
 
 
-async def load_disabled_extension_list() -> None:
-    """Update list of extensions that have been explicitly disabled"""
-    inactive_extensions = await get_installed_extensions(active=False)
-    settings.lnbits_deactivated_extensions.update([e.id for e in inactive_extensions])
+@users.command("new")
+@click.option("-u", "--username", required=True, help="Username.")
+@click.option("-p", "--password", required=True, help="Password.")
+@coro
+async def create_user(username: str, password: str):
+    """Create a new user bypassing the system 'new_accounts_allowed' rules"""
+    account = Account(
+        id=uuid4().hex,
+        username=username,
+    )
+    account.hash_password(password)
+    user = await create_user_account_no_ckeck(account)
+    click.echo(f"User '{user.username}' created. Id: '{user.id}'")
+
+
+@users.command("cleanup-accounts")
+@click.argument("days", type=int, required=False)
+@coro
+async def database_cleanup_accounts(days: Optional[int] = None):
+    """Delete all accounts that have no wallets"""
+    async with core_db.connect() as conn:
+        delta = days or settings.cleanup_wallets_days
+        delta = delta * 24 * 60 * 60
+        await delete_accounts_no_wallets(delta, conn)
 
 
 @extensions.command("list")
@@ -328,8 +320,10 @@ async def extensions_list():
     from lnbits.app import build_all_installed_extensions_list
 
     for ext in await build_all_installed_extensions_list():
-        assert ext.installed_release, f"Extension {ext.id} has no installed_release"
-        click.echo(f"  - {ext.id} ({ext.installed_release.version})")
+        assert (
+            ext.meta and ext.meta.installed_release
+        ), f"Extension {ext.id} has no installed_release"
+        click.echo(f"  - {ext.id} ({ext.meta.installed_release.version})")
 
 
 @extensions.command("update")
@@ -378,19 +372,24 @@ async def extensions_update(
     if extension and all_extensions:
         click.echo("Only one of extension ID or the '--all' flag must be specified")
         return
-    if url and not _is_url(url):
+    if url and not is_valid_url(url):
         click.echo(f"Invalid '--url' option value: {url}")
         return
 
     if not await _can_run_operation(url):
         return
 
+    upgrades_dir = settings.lnbits_extensions_upgrade_path
+    Path(upgrades_dir).mkdir(parents=True, exist_ok=True)
+    sys.path.append(str(upgrades_dir))
+
     if extension:
         await update_extension(extension, repo_index, source_repo, url, admin_user)
         return
 
-    click.echo("Updating all extensions...")
+    click.echo("Updating all extensions:")
     installed_extensions = await get_installed_extensions()
+    click.echo(f"   {[e.id for e in installed_extensions]}")
     updated_extensions = []
     for e in installed_extensions:
         try:
@@ -452,7 +451,7 @@ async def extensions_install(
 ):
     """Install a extension"""
     click.echo(f"Installing {extension}... {repo_index}")
-    if url and not _is_url(url):
+    if url and not is_valid_url(url):
         click.echo(f"Invalid '--url' option value: {url}")
         return
 
@@ -480,7 +479,7 @@ async def extensions_uninstall(
     """Uninstall a extension"""
     click.echo(f"Uninstalling '{extension}'...")
 
-    if url and not _is_url(url):
+    if url and not is_valid_url(url):
         click.echo(f"Invalid '--url' option value: {url}")
         return
 
@@ -497,6 +496,62 @@ async def extensions_uninstall(
         return False, str(ex)
 
 
+@encrypt.command("macaroon")
+def encrypt_macaroon():
+    """Encrypts a macaroon (LND wallets)"""
+    _macaroon = getpass("Enter macaroon: ")
+    try:
+        macaroon = load_macaroon(_macaroon)
+    except Exception as ex:
+        click.echo(f"Error loading macaroon: {ex}")
+        return
+    key = getpass("Enter encryption key: ")
+    aes = AESCipher(key.encode())
+    try:
+        encrypted_macaroon = aes.encrypt(bytes.fromhex(macaroon))
+    except Exception as ex:
+        click.echo(f"Error encrypting macaroon: {ex}")
+        return
+    click.echo("Encrypted macaroon: ")
+    click.echo(encrypted_macaroon)
+
+
+@encrypt.command("aes")
+@click.option("-p", "--payload", required=True, help="Payload to encrypt.")
+@click.option(
+    "-u", "--urlsafe", is_flag=True, required=False, help="Urlsafe b64encode."
+)
+def encrypt_aes(payload: str, urlsafe: bool = False):
+    """AES encrypts a payload"""
+    key = getpass("Enter encryption key: ")
+    aes = AESCipher(key.encode())
+    try:
+        encrypted = aes.encrypt(payload.encode(), urlsafe=urlsafe)
+    except Exception as ex:
+        click.echo(f"Error encrypting payload: {ex}")
+        return
+    click.echo("Encrypted payload: ")
+    click.echo(encrypted)
+
+
+@decrypt.command("aes")
+@click.option("-p", "--payload", required=True, help="Payload to decrypt.")
+@click.option(
+    "-u", "--urlsafe", is_flag=True, required=False, help="Urlsafe b64decode."
+)
+def decrypt_aes(payload: str, urlsafe: bool = False):
+    """AES decrypts a payload"""
+    key = getpass("Enter encryption key: ")
+    aes = AESCipher(key.encode())
+    try:
+        decrypted = aes.decrypt(payload, urlsafe=urlsafe)
+    except Exception as ex:
+        click.echo(f"Error decrypting payload: {ex}")
+        return
+    click.echo("Decrypted payload: ")
+    click.echo(decrypted)
+
+
 def main():
     """main function"""
     lnbits_cli()
@@ -512,7 +567,7 @@ async def install_extension(
     source_repo: Optional[str] = None,
     url: Optional[str] = None,
     admin_user: Optional[str] = None,
-) -> Tuple[bool, str]:
+) -> tuple[bool, str]:
     try:
         release = await _select_release(extension, repo_index, source_repo)
         if not release:
@@ -541,7 +596,7 @@ async def update_extension(
     source_repo: Optional[str] = None,
     url: Optional[str] = None,
     admin_user: Optional[str] = None,
-) -> Tuple[bool, str]:
+) -> tuple[bool, str]:
     try:
         click.echo(f"Updating '{extension}' extension.")
         installed_ext = await get_installed_extension(extension)
@@ -554,7 +609,7 @@ async def update_extension(
         click.echo(f"Current '{extension}' version: {installed_ext.installed_version}.")
 
         assert (
-            installed_ext.installed_release
+            installed_ext.meta and installed_ext.meta.installed_release
         ), "Cannot find previously installed release. Please uninstall first."
 
         release = await _select_release(extension, repo_index, source_repo)
@@ -562,7 +617,7 @@ async def update_extension(
             return False, "No release selected."
         if (
             release.version == installed_ext.installed_version
-            and release.source_repo == installed_ext.installed_release.source_repo
+            and release.source_repo == installed_ext.meta.installed_release.source_repo
         ):
             click.echo(f"Extension '{extension}' already up to date.")
             return False, "Already up to date"
@@ -661,7 +716,7 @@ async def _call_install_extension(
             )
             resp.raise_for_status()
     else:
-        await api_install_extension(data, User(id="mock_id"))
+        await api_install_extension(data)
 
 
 async def _call_uninstall_extension(
@@ -675,7 +730,7 @@ async def _call_uninstall_extension(
             )
             resp.raise_for_status()
     else:
-        await api_uninstall_extension(extension, User(id="mock_id"))
+        await api_uninstall_extension(extension)
 
 
 async def _can_run_operation(url) -> bool:
@@ -694,7 +749,7 @@ async def _can_run_operation(url) -> bool:
     elif url:
         click.echo(
             "The option '--url' has been provided,"
-            + f" but no server found runnint at '{url}'"
+            f" but no server found running at '{url}'"
         )
         return False
 
@@ -709,11 +764,3 @@ async def _is_lnbits_started(url: Optional[str]):
             return True
     except Exception:
         return False
-
-
-def _is_url(url):
-    try:
-        result = urlparse(url)
-        return all([result.scheme, result.netloc])
-    except ValueError:
-        return False
