@@ -1,13 +1,15 @@
-import datetime
+import json
 from time import time
+from typing import Any
 
 from loguru import logger
 from sqlalchemy.exc import OperationalError
 
 from lnbits import bolt11
+from lnbits.db import Connection
 
 
-async def m000_create_migrations_table(db):
+async def m000_create_migrations_table(db: Connection):
     await db.execute(
         """
     CREATE TABLE IF NOT EXISTS dbversions (
@@ -18,7 +20,7 @@ async def m000_create_migrations_table(db):
     )
 
 
-async def m001_initial(db):
+async def m001_initial(db: Connection):
     """
     Initial LNbits tables.
     """
@@ -87,7 +89,7 @@ async def m001_initial(db):
     )
 
 
-async def m002_add_fields_to_apipayments(db):
+async def m002_add_fields_to_apipayments(db: Connection):
     """
     Adding fields to apipayments for better accounting,
     and renaming payhash to checking_id since that is what it really is.
@@ -100,9 +102,8 @@ async def m002_add_fields_to_apipayments(db):
         await db.execute("ALTER TABLE apipayments ADD COLUMN bolt11 TEXT")
         await db.execute("ALTER TABLE apipayments ADD COLUMN extra TEXT")
 
-        import json
-
-        rows = await (await db.execute("SELECT * FROM apipayments")).fetchall()
+        result = await db.execute("SELECT * FROM apipayments")
+        rows = result.mappings().all()
         for row in rows:
             if not row["memo"] or not row["memo"].startswith("#"):
                 continue
@@ -113,15 +114,15 @@ async def m002_add_fields_to_apipayments(db):
                     new = row["memo"][len(prefix) :]
                     await db.execute(
                         """
-                        UPDATE apipayments SET extra = ?, memo = ?
-                        WHERE checking_id = ? AND memo = ?
+                        UPDATE apipayments SET extra = :extra, memo = :memo1
+                        WHERE checking_id = :checking_id AND memo = :memo2
                         """,
-                        (
-                            json.dumps({"tag": ext}),
-                            new,
-                            row["checking_id"],
-                            row["memo"],
-                        ),
+                        {
+                            "extra": json.dumps({"tag": ext}),
+                            "memo1": new,
+                            "checking_id": row["checking_id"],
+                            "memo2": row["memo"],
+                        },
                     )
                     break
     except OperationalError:
@@ -132,7 +133,7 @@ async def m002_add_fields_to_apipayments(db):
         pass
 
 
-async def m003_add_invoice_webhook(db):
+async def m003_add_invoice_webhook(db: Connection):
     """
     Special column for webhook endpoints that can be assigned
     to each different invoice.
@@ -142,7 +143,7 @@ async def m003_add_invoice_webhook(db):
     await db.execute("ALTER TABLE apipayments ADD COLUMN webhook_status TEXT")
 
 
-async def m004_ensure_fees_are_always_negative(db):
+async def m004_ensure_fees_are_always_negative(db: Connection):
     """
     Use abs() so wallet backends don't have to care about the sign of the fees.
     """
@@ -167,7 +168,7 @@ async def m004_ensure_fees_are_always_negative(db):
     )
 
 
-async def m005_balance_check_balance_notify(db):
+async def m005_balance_check_balance_notify(db: Connection):
     """
     Keep track of balanceCheck-enabled lnurl-withdrawals to be consumed by an
     LNbits wallet and of balanceNotify URLs supplied by users to empty their wallets.
@@ -197,7 +198,7 @@ async def m005_balance_check_balance_notify(db):
     )
 
 
-async def m006_add_invoice_expiry_to_apipayments(db):
+async def m006_add_invoice_expiry_to_apipayments(db: Connection):
     """
     Adds invoice expiry column to apipayments.
     """
@@ -207,24 +208,23 @@ async def m006_add_invoice_expiry_to_apipayments(db):
         pass
 
 
-async def m007_set_invoice_expiries(db):
+async def m007_set_invoice_expiries(db: Connection):
     """
     Precomputes invoice expiry for existing pending incoming payments.
     """
     try:
-        rows = await (
-            await db.execute(
-                f"""
-                SELECT bolt11, checking_id
-                FROM apipayments
-                WHERE pending = true
-                AND amount > 0
-                AND bolt11 IS NOT NULL
-                AND expiry IS NULL
-                AND time < {db.timestamp_now}
-                """
-            )
-        ).fetchall()
+        result = await db.execute(
+            f"""
+            SELECT bolt11, checking_id
+            FROM apipayments
+            WHERE pending = true
+            AND amount > 0
+            AND bolt11 IS NOT NULL
+            AND expiry IS NULL
+            AND time < {db.timestamp_now}
+            """
+        )
+        rows = result.mappings().all()
         if len(rows):
             logger.info(f"Migration: Checking expiry of {len(rows)} invoices")
         for i, (
@@ -236,22 +236,17 @@ async def m007_set_invoice_expiries(db):
                 if invoice.expiry is None:
                     continue
 
-                expiration_date = datetime.datetime.fromtimestamp(
-                    invoice.date + invoice.expiry
-                )
+                expiration_date = invoice.date + invoice.expiry
                 logger.info(
                     f"Migration: {i+1}/{len(rows)} setting expiry of invoice"
                     f" {invoice.payment_hash} to {expiration_date}"
                 )
                 await db.execute(
-                    """
-                    UPDATE apipayments SET expiry = ?
-                    WHERE checking_id = ? AND amount > 0
+                    f"""
+                    UPDATE apipayments SET expiry = {db.timestamp_placeholder('expiry')}
+                    WHERE checking_id = :checking_id AND amount > 0
                     """,
-                    (
-                        db.datetime_to_timestamp(expiration_date),
-                        checking_id,
-                    ),
+                    {"expiry": expiration_date, "checking_id": checking_id},
                 )
             except Exception:
                 continue
@@ -263,7 +258,7 @@ async def m007_set_invoice_expiries(db):
         pass
 
 
-async def m008_create_admin_settings_table(db):
+async def m008_create_admin_settings_table(db: Connection):
     await db.execute(
         """
         CREATE TABLE IF NOT EXISTS settings (
@@ -274,7 +269,7 @@ async def m008_create_admin_settings_table(db):
     )
 
 
-async def m009_create_tinyurl_table(db):
+async def m009_create_tinyurl_table(db: Connection):
     await db.execute(
         f"""
         CREATE TABLE IF NOT EXISTS tiny_url (
@@ -288,7 +283,7 @@ async def m009_create_tinyurl_table(db):
     )
 
 
-async def m010_create_installed_extensions_table(db):
+async def m010_create_installed_extensions_table(db: Connection):
     await db.execute(
         """
         CREATE TABLE IF NOT EXISTS installed_extensions (
@@ -305,7 +300,7 @@ async def m010_create_installed_extensions_table(db):
     )
 
 
-async def m011_optimize_balances_view(db):
+async def m011_optimize_balances_view(db: Connection):
     """
     Make the calculation of the balance a single aggregation
     over the payments table instead of 2.
@@ -322,7 +317,7 @@ async def m011_optimize_balances_view(db):
     )
 
 
-async def m012_add_currency_to_wallet(db):
+async def m012_add_currency_to_wallet(db: Connection):
     await db.execute(
         """
         ALTER TABLE wallets ADD COLUMN currency TEXT
@@ -330,7 +325,7 @@ async def m012_add_currency_to_wallet(db):
     )
 
 
-async def m013_add_deleted_to_wallets(db):
+async def m013_add_deleted_to_wallets(db: Connection):
     """
     Adds deleted column to wallets.
     """
@@ -342,35 +337,39 @@ async def m013_add_deleted_to_wallets(db):
         pass
 
 
-async def m014_set_deleted_wallets(db):
+async def m014_set_deleted_wallets(db: Connection):
     """
     Sets deleted column to wallets.
     """
     try:
-        rows = await (
-            await db.execute(
-                """
-                SELECT *
-                FROM wallets
-                WHERE user LIKE 'del:%'
-                AND adminkey LIKE 'del:%'
-                AND inkey LIKE 'del:%'
-                """
-            )
-        ).fetchall()
+        result = await db.execute(
+            """
+            SELECT *
+            FROM wallets
+            WHERE user LIKE 'del:%'
+            AND adminkey LIKE 'del:%'
+            AND inkey LIKE 'del:%'
+            """
+        )
+        rows = result.mappings().all()
 
         for row in rows:
             try:
-                user = row[2].split(":")[1]
-                adminkey = row[3].split(":")[1]
-                inkey = row[4].split(":")[1]
+                user = row["user"].split(":")[1]
+                adminkey = row["adminkey"].split(":")[1]
+                inkey = row["inkey"].split(":")[1]
                 await db.execute(
                     """
                     UPDATE wallets SET
-                    "user" = ?, adminkey = ?, inkey = ?, deleted = true
-                    WHERE id = ?
+                    "user" = :user, adminkey = :adminkey, inkey = :inkey, deleted = true
+                    WHERE id = :wallet
                     """,
-                    (user, adminkey, inkey, row[0]),
+                    {
+                        "user": user,
+                        "adminkey": adminkey,
+                        "inkey": inkey,
+                        "wallet": row.get("id"),
+                    },
                 )
             except Exception:
                 continue
@@ -382,7 +381,7 @@ async def m014_set_deleted_wallets(db):
         pass
 
 
-async def m015_create_push_notification_subscriptions_table(db):
+async def m015_create_push_notification_subscriptions_table(db: Connection):
     await db.execute(
         f"""
         CREATE TABLE IF NOT EXISTS webpush_subscriptions (
@@ -397,7 +396,7 @@ async def m015_create_push_notification_subscriptions_table(db):
     )
 
 
-async def m016_add_username_column_to_accounts(db):
+async def m016_add_username_column_to_accounts(db: Connection):
     """
     Adds username column to accounts.
     """
@@ -408,7 +407,7 @@ async def m016_add_username_column_to_accounts(db):
         pass
 
 
-async def m017_add_timestamp_columns_to_accounts_and_wallets(db):
+async def m017_add_timestamp_columns_to_accounts_and_wallets(db: Connection):
     """
     Adds created_at and updated_at column to accounts and wallets.
     """
@@ -456,17 +455,17 @@ async def m017_add_timestamp_columns_to_accounts_and_wallets(db):
         now = int(time())
         await db.execute(
             f"""
-            UPDATE wallets SET created_at = {db.timestamp_placeholder}
+            UPDATE wallets SET created_at = {db.timestamp_placeholder('now')}
             WHERE created_at IS NULL
             """,
-            (now,),
+            {"now": now},
         )
         await db.execute(
             f"""
-            UPDATE accounts SET created_at = {db.timestamp_placeholder}
+            UPDATE accounts SET created_at = {db.timestamp_placeholder('now')}
             WHERE created_at IS NULL
             """,
-            (now,),
+            {"now": now},
         )
 
     except OperationalError as exc:
@@ -474,7 +473,7 @@ async def m017_add_timestamp_columns_to_accounts_and_wallets(db):
         pass
 
 
-async def m018_balances_view_exclude_deleted(db):
+async def m018_balances_view_exclude_deleted(db: Connection):
     """
     Make deleted wallets not show up in the balances view.
     """
@@ -494,7 +493,7 @@ async def m018_balances_view_exclude_deleted(db):
     )
 
 
-async def m019_balances_view_based_on_wallets(db):
+async def m019_balances_view_based_on_wallets(db: Connection):
     """
     Make deleted wallets not show up in the balances view.
     Important for querying whole lnbits balances.
@@ -515,8 +514,212 @@ async def m019_balances_view_based_on_wallets(db):
     )
 
 
-async def m020_add_column_column_to_user_extensions(db):
+async def m020_add_column_column_to_user_extensions(db: Connection):
     """
     Adds extra column to user extensions.
     """
     await db.execute("ALTER TABLE extensions ADD COLUMN extra TEXT")
+
+
+async def m021_add_success_failed_to_apipayments(db: Connection):
+    """
+    Adds success and failed columns to apipayments.
+    """
+    await db.execute("ALTER TABLE apipayments ADD COLUMN status TEXT DEFAULT 'pending'")
+    #  set all not pending to success true, failed payments were deleted until now
+    await db.execute("UPDATE apipayments SET status = 'success' WHERE NOT pending")
+
+    await db.execute("DROP VIEW balances")
+    await db.execute(
+        """
+        CREATE VIEW balances AS
+        SELECT apipayments.wallet,
+               SUM(apipayments.amount - ABS(apipayments.fee)) AS balance
+        FROM wallets
+        LEFT JOIN apipayments ON apipayments.wallet = wallets.id
+        WHERE (wallets.deleted = false OR wallets.deleted is NULL)
+        AND (
+            (apipayments.status = 'success' AND apipayments.amount > 0)
+            OR (apipayments.status IN ('success', 'pending') AND apipayments.amount < 0)
+        )
+        GROUP BY apipayments.wallet
+    """
+    )
+
+
+async def m022_add_pubkey_to_accounts(db: Connection):
+    """
+    Adds pubkey column to accounts.
+    """
+    try:
+        await db.execute("ALTER TABLE accounts ADD COLUMN pubkey TEXT")
+    except OperationalError:
+        pass
+
+
+async def m023_add_column_column_to_apipayments(db: Connection):
+    """
+    renames hash to payment_hash and drops unused index
+    """
+    await db.execute("DROP INDEX by_hash")
+    await db.execute("ALTER TABLE apipayments RENAME COLUMN hash TO payment_hash")
+    await db.execute("ALTER TABLE apipayments RENAME COLUMN wallet TO wallet_id")
+    await db.execute("ALTER TABLE accounts RENAME COLUMN pass TO password_hash")
+
+    await db.execute("CREATE INDEX by_hash ON apipayments (payment_hash)")
+
+
+async def m024_drop_pending(db: Connection):
+    await db.execute("ALTER TABLE apipayments DROP COLUMN pending")
+
+
+async def m025_refresh_view(db: Connection):
+    await db.execute("DROP VIEW balances")
+    await db.execute(
+        """
+        CREATE VIEW balances AS
+        SELECT apipayments.wallet_id,
+               SUM(apipayments.amount - ABS(apipayments.fee)) AS balance
+        FROM wallets
+        LEFT JOIN apipayments ON apipayments.wallet_id = wallets.id
+        WHERE (wallets.deleted = false OR wallets.deleted is NULL)
+        AND (
+            (apipayments.status = 'success' AND apipayments.amount > 0)
+            OR (apipayments.status IN ('success', 'pending') AND apipayments.amount < 0)
+        )
+        GROUP BY apipayments.wallet_id
+    """
+    )
+
+
+async def m026_update_payment_table(db: Connection):
+    await db.execute("ALTER TABLE apipayments ADD COLUMN tag TEXT")
+    await db.execute("ALTER TABLE apipayments ADD COLUMN extension TEXT")
+    await db.execute("ALTER TABLE apipayments ADD COLUMN created_at TIMESTAMP")
+    await db.execute("ALTER TABLE apipayments ADD COLUMN updated_at TIMESTAMP")
+
+
+async def m027_update_apipayments_data(db: Connection):
+    result = None
+    try:
+        result = await db.execute("SELECT * FROM apipayments LIMIT 100")
+    except Exception as exc:
+        logger.warning("Could not select, trying again after cache cleared.")
+        logger.debug(exc)
+        await db.execute("COMMIT")
+
+    offset = 0
+    limit = 1000
+    payments: list[dict[Any, Any]] = []
+    logger.info("Updating payments")
+    while len(payments) > 0 or offset == 0:
+        logger.info(f"Updating {offset} to {offset+limit}")
+
+        result = await db.execute(
+            f"SELECT * FROM apipayments  ORDER BY time LIMIT {limit} OFFSET {offset}"
+        )
+        payments = result.mappings().all()
+        logger.info(f"Payments count: {len(payments)}")
+
+        for payment in payments:
+            tag = None
+            created_at = payment.get("time")
+            if payment.get("extra"):
+                extra = json.loads(str(payment.get("extra")))
+                tag = extra.get("tag")
+            tsph = db.timestamp_placeholder("created_at")
+            await db.execute(
+                f"""
+                UPDATE apipayments
+                SET tag = :tag, created_at = {tsph}, updated_at = {tsph}
+                WHERE checking_id = :checking_id
+                """,
+                {
+                    "tag": tag,
+                    "created_at": created_at,
+                    "checking_id": payment.get("checking_id"),
+                },
+            )
+        offset += limit
+    logger.info("Payments updated")
+
+
+async def m028_update_settings(db: Connection):
+
+    await db.execute(
+        """
+        CREATE TABLE IF NOT EXISTS system_settings (
+            id TEXT PRIMARY KEY,
+            value TEXT,
+            tag TEXT NOT NULL DEFAULT 'core',
+
+            UNIQUE (id, tag)
+        );
+    """
+    )
+
+    async def _insert_key_value(id_: str, value: Any):
+        await db.execute(
+            """
+            INSERT INTO system_settings (id, value, tag)
+            VALUES (:id, :value, :tag)
+            """,
+            {"id": id_, "value": json.dumps(value), "tag": "core"},
+        )
+
+    row: dict = await db.fetchone("SELECT * FROM settings")
+    if row:
+        await _insert_key_value("super_user", row["super_user"])
+        editable_settings = json.loads(row["editable_settings"])
+
+        for key, value in editable_settings.items():
+            await _insert_key_value(key, value)
+
+    await db.execute("drop table settings")
+
+
+async def m029_create_audit_table(db: Connection):
+    await db.execute(
+        f"""
+        CREATE TABLE IF NOT EXISTS audit (
+            component TEXT,
+            ip_address TEXT,
+            user_id TEXT,
+            path TEXT,
+            request_type TEXT,
+            request_method TEXT,
+            request_details TEXT,
+            response_code TEXT,
+            duration REAL NOT NULL,
+            delete_at TIMESTAMP,
+            created_at TIMESTAMP NOT NULL DEFAULT {db.timestamp_now}
+        );
+        """
+    )
+
+
+async def m030_add_user_api_tokens_column(db: Connection):
+    await db.execute(
+        """
+        ALTER TABLE accounts ADD COLUMN access_control_list TEXT
+        """
+    )
+
+
+async def m031_add_color_and_icon_to_wallets(db: Connection):
+    """
+    Adds icon and color columns to wallets.
+    """
+    await db.execute("ALTER TABLE wallets ADD COLUMN extra TEXT")
+
+
+async def m032_add_external_id_to_accounts(db: Connection):
+    """
+    Adds external_id column to accounts.
+    Used for external account linking.
+    """
+    await db.execute("ALTER TABLE accounts ADD COLUMN external_id TEXT")
+
+
+async def m033_update_payment_table(db: Connection):
+    await db.execute("ALTER TABLE apipayments ADD COLUMN fiat_provider TEXT")
